package ca.mcgill.sel.workflow.samples;

import ca.mcgill.sel.workflow.CustomizableInputData;
import ca.mcgill.sel.workflow.CustomizableNode;
import ca.mcgill.sel.workflow.CustomizableOutputData;
import ca.mcgill.sel.workflow.EndNode;
import ca.mcgill.sel.workflow.InputData;
import ca.mcgill.sel.workflow.InputNode;
import ca.mcgill.sel.workflow.LocalContext;
import ca.mcgill.sel.workflow.StartupNode;
import ca.mcgill.sel.workflow.samples.AbstractWorkflowDemo;
import ca.mcgill.sel.workflow.samples.InputWorkflow;

public class InputOutputWorkflow extends AbstractWorkflowDemo {
    
    private class SomeInputData extends CustomizableInputData {

        private String data;
        
        public SomeInputData(String senderName, String queueName, String data) {
            super(senderName, queueName);
            
            this.data = data;
        }
        
        public String getData() {
            return data;
        }
        
    }
    
    private class ProcessInputNode extends CustomizableNode {
        
        @Override
        public void execute(LocalContext c) {
            InputData inputData = c.getMyInput();
            SomeInputData concreteInputData = (SomeInputData) inputData;
            System.out.println("Input received: " + concreteInputData.data);
        }
        
    }
    
    private class AskForInformationOutput extends CustomizableOutputData {
        
        private String message;
        
        public AskForInformationOutput(String message) {
            this.message = message;
        }
        
        public String getMessage() {
            return message;
        }
        
    }
    
    private class AskForInformation extends CustomizableNode {

        @Override
        public void execute(LocalContext c) {
            AskForInformationOutput output = new AskForInformationOutput("Gimme some information pls!");
            // TODO: simply output it on the console.
            System.out.println("Send over network: " + output.message);
            output.send("unknownHost");
        }
        
    }

    public InputOutputWorkflow(LocalContext context) {
        super(context);
    }

    @Override
    public void initNodes() {
        StartupNode startupNode = new StartupNode();
        EndNode endNode = new EndNode();
        
        AskForInformation requestNode = new AskForInformation();
        
        InputNode inputNode = new InputNode();
        ProcessInputNode processNode = new ProcessInputNode();
        
        workflow.addNode(startupNode);
        workflow.addNode(inputNode);
        workflow.addNode(requestNode);
        workflow.addNode(processNode);
        workflow.addNode(endNode);
        
        startupNode.setNextNode(inputNode);
        inputNode.setNextNode(requestNode);
        requestNode.setNextNode(processNode);
        processNode.setNextNode(endNode);
        
        workflow.addStartupNode(startupNode);
    }

    public static void main(String[] args) throws InterruptedException {
        LocalContext context = new LocalContext();
        InputOutputWorkflow workflow = new InputOutputWorkflow(context);
        workflow.executeWorkflow();
        
        Thread.sleep(2000);
        
        SomeInputData inputData = workflow.new SomeInputData("unsure", "unsure", "teh data");
        inputData.execute();
    }

}
